@@ -1,7 +1,6 @@
 /* GNU m4 -- A simple macro processor
 
-   Copyright (C) 1989, 1990, 1991, 1992, 1993, 1994, 2004, 2005, 2006, 2007
-   Free Software Foundation, Inc.
+   Copyright (C) 1989-1994, 2004-2011 Free Software Foundation, Inc.
 
    This file is part of GNU M4.
 
@@ -25,11 +24,18 @@
 #include <limits.h>
 #include <signal.h>
 
+#include "c-stack.h"
+#include "ignore-value.h"
+#include "progname.h"
 #include "version-etc.h"
 
+#ifdef DEBUG_STKOVF
+# include "assert.h"
+#endif
+
 #define AUTHORS "Rene' Seindal"
 
-static void usage (int);
+static void usage (int) M4_GNUC_NORETURN;
 
 /* Enable sync output for /lib/cpp (-s).  */
 int sync_output = 0;
@@ -66,9 +72,6 @@ int nesting_limit = 1024;
 const char *user_word_regexp = "";
 #endif
 
-/* The name this program was run with. */
-const char *program_name;
-
 /* Global catchall for any errors that should affect final error status, but
    where we try to continue execution in the meantime.  */
 int retcode;
@@ -76,7 +79,7 @@ int retcode;
 struct macro_definition
 {
   struct macro_definition *next;
-  int code;			/* D, U, s, t, or '\1' */
+  int code; /* D, U, s, t, '\1', or DEBUGFILE_OPTION.  */
   const char *arg;
 };
 typedef struct macro_definition macro_definition;
@@ -93,7 +96,7 @@ m4_error (int status, int errnum, const char *format, ...)
   va_list args;
   va_start (args, format);
   verror_at_line (status, errnum, current_line ? current_file : NULL,
-		  current_line, format, args);
+                  current_line, format, args);
   if (fatal_warnings && ! retcode)
     retcode = EXIT_FAILURE;
 }
@@ -104,7 +107,7 @@ m4_error (int status, int errnum, const char *format, ...)
 
 void
 m4_error_at_line (int status, int errnum, const char *file, int line,
-		  const char *format, ...)
+                  const char *format, ...)
 {
   va_list args;
   va_start (args, format);
@@ -113,20 +116,55 @@ m4_error_at_line (int status, int errnum, const char *file, int line,
     retcode = EXIT_FAILURE;
 }
 
-#ifdef USE_STACKOVF
+#ifndef SIGBUS
+# define SIGBUS SIGILL
+#endif
 
-/*---------------------------------------.
-| Tell user stack overflowed and abort.	 |
-`---------------------------------------*/
+#ifndef NSIG
+# ifndef MAX
+#  define MAX(a,b) ((a) < (b) ? (b) : (a))
+# endif
+# define NSIG (MAX (SIGABRT, MAX (SIGILL, MAX (SIGFPE,  \
+                                               MAX (SIGSEGV, SIGBUS)))) + 1)
+#endif
 
+/* Pre-translated messages for program errors.  Do not translate in
+   the signal handler, since gettext and strsignal are not
+   async-signal-safe.  */
+static const char * volatile program_error_message;
+static const char * volatile signal_message[NSIG];
+
+/* Print a nicer message about any programmer errors, then exit.  This
+   must be aysnc-signal safe, since it is executed as a signal
+   handler.  If SIGNO is zero, this represents a stack overflow; in
+   that case, we return to allow c_stack_action to handle things.  */
 static void
-stackovf_handler (void)
+fault_handler (int signo)
 {
-  M4ERROR ((EXIT_FAILURE, 0,
-	    "ERROR: stack overflow.  (Infinite define recursion?)"));
+  if (signo)
+    {
+      /* POSIX states that reading static memory is, in general, not
+         async-safe.  However, the static variables that we read are
+         never modified once this handler is installed, so this
+         particular usage is safe.  And it seems an oversight that
+         POSIX claims strlen is not async-safe.  Ignore write
+         failures, since we will exit with non-zero status anyway.  */
+#define WRITE(f, b, l) ignore_value (write (f, b, l))
+      WRITE (STDERR_FILENO, program_name, strlen (program_name));
+      WRITE (STDERR_FILENO, ": ", 2);
+      WRITE (STDERR_FILENO, program_error_message,
+             strlen (program_error_message));
+      if (signal_message[signo])
+        {
+          WRITE (STDERR_FILENO, ": ", 2);
+          WRITE (STDERR_FILENO, signal_message[signo],
+                 strlen (signal_message[signo]));
+        }
+      WRITE (STDERR_FILENO, "\n", 1);
+#undef WRITE
+      _exit (EXIT_INTERNAL_ERROR);
+    }
 }
-
-#endif /* USE_STACKOV */
 
 
 /*---------------------------------------------.
@@ -156,13 +194,13 @@ Operation modes:\n\
 ", stdout);
       xprintf ("\
   -E, --fatal-warnings         once: warnings become errors, twice: stop\n\
-                               execution at first error\n\
+                                 execution at first error\n\
   -i, --interactive            unbuffer output, ignore interrupts\n\
   -P, --prefix-builtins        force a `m4_' prefix to all builtins\n\
   -Q, --quiet, --silent        suppress some warnings for builtins\n\
       --warn-macro-sequence[=REGEXP]\n\
                                warn if macro definition matches REGEXP,\n\
-                               default %s\n\
+                                 default %s\n\
 ", DEFAULT_MACRO_SEQUENCE);
 #ifdef ENABLE_CHANGEWORD
       fputs ("\
@@ -177,15 +215,16 @@ Preprocessor features:\n\
   -s, --synclines              generate `#line NUM \"FILE\"' lines\n\
   -U, --undefine=NAME          undefine NAME\n\
 ", stdout);
-      fputs ("\
-\n\
+      puts ("");
+      xprintf (_("\
 Limits control:\n\
+  -g, --gnu                    override -G to re-enable GNU extensions\n\
   -G, --traditional            suppress all GNU extensions\n\
   -H, --hashsize=PRIME         set symbol lookup hash table size [509]\n\
-  -L, --nesting-limit=NUMBER   change artificial nesting limit [1024]\n\
-", stdout);
+  -L, --nesting-limit=NUMBER   change nesting limit, 0 for unlimited [%d]\n\
+"), nesting_limit);
+      puts ("");
       fputs ("\
-\n\
 Frozen state files:\n\
   -F, --freeze-state=FILE      produce a frozen state on FILE at end\n\
   -R, --reload-state=FILE      reload a frozen state from FILE at start\n\
@@ -194,7 +233,8 @@ Frozen state files:\n\
 \n\
 Debugging:\n\
   -d, --debug[=FLAGS]          set debug level (no FLAGS implies `aeq')\n\
-      --debugfile=FILE         redirect debug and trace output\n\
+      --debugfile[=FILE]       redirect debug and trace output to FILE\n\
+                                 (default stderr, discard if empty string)\n\
   -l, --arglength=NUM          restrict macro tracing size\n\
   -t, --trace=NAME             trace NAME when it is defined\n\
 ", stdout);
@@ -223,7 +263,7 @@ of directories included after any specified by `-I'.\n\
 Exit status is 0 for success, 1 for failure, 63 for frozen file version\n\
 mismatch, or whatever value was passed to the m4exit macro.\n\
 ", stdout);
-      xprintf ("\nReport bugs to <%s>.\n", PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -236,12 +276,12 @@ mismatch, or whatever value was passed to the m4exit macro.\n\
    non-character as a pseudo short option, starting with CHAR_MAX + 1.  */
 enum
 {
-  DEBUGFILE_OPTION = CHAR_MAX + 1,	/* no short opt */
-  DIVERSIONS_OPTION,			/* not quite -N, because of message */
-  WARN_MACRO_SEQUENCE_OPTION,		/* no short opt */
+  DEBUGFILE_OPTION = CHAR_MAX + 1,      /* no short opt */
+  DIVERSIONS_OPTION,                    /* not quite -N, because of message */
+  WARN_MACRO_SEQUENCE_OPTION,           /* no short opt */
 
-  HELP_OPTION,				/* no short opt */
-  VERSION_OPTION			/* no short opt */
+  HELP_OPTION,                          /* no short opt */
+  VERSION_OPTION                        /* no short opt */
 };
 
 static const struct option long_options[] =
@@ -252,6 +292,7 @@ static const struct option long_options[] =
   {"error-output", required_argument, NULL, 'o'}, /* FIXME: deprecate in 2.0 */
   {"fatal-warnings", no_argument, NULL, 'E'},
   {"freeze-state", required_argument, NULL, 'F'},
+  {"gnu", no_argument, NULL, 'g'},
   {"hashsize", required_argument, NULL, 'H'},
   {"include", required_argument, NULL, 'I'},
   {"interactive", no_argument, NULL, 'i'},
@@ -266,7 +307,7 @@ static const struct option long_options[] =
   {"undefine", required_argument, NULL, 'U'},
   {"word-regexp", required_argument, NULL, 'W'},
 
-  {"debugfile", required_argument, NULL, DEBUGFILE_OPTION},
+  {"debugfile", optional_argument, NULL, DEBUGFILE_OPTION},
   {"diversions", required_argument, NULL, DIVERSIONS_OPTION},
   {"warn-macro-sequence", optional_argument, NULL, WARN_MACRO_SEQUENCE_OPTION},
 
@@ -281,12 +322,12 @@ static const struct option long_options[] =
 static void
 process_file (const char *name)
 {
-  if (strcmp (name, "-") == 0)
+  if (STREQ (name, "-"))
     {
       /* If stdin is a terminal, we want to allow 'm4 - file -'
-	 to read input from stdin twice, like GNU cat.  Besides,
-	 there is no point closing stdin before wrapped text, to
-	 minimize bugs in syscmd called from wrapped text.  */
+         to read input from stdin twice, like GNU cat.  Besides,
+         there is no point closing stdin before wrapped text, to
+         minimize bugs in syscmd called from wrapped text.  */
       push_file (stdin, "stdin", false);
     }
   else
@@ -294,13 +335,13 @@ process_file (const char *name)
       char *full_name;
       FILE *fp = m4_path_search (name, &full_name);
       if (fp == NULL)
-	{
-	  error (0, errno, "%s", name);
-	  /* Set the status to EXIT_FAILURE, even though we
-	     continue to process files after a missing file.  */
-	  retcode = EXIT_FAILURE;
-	  return;
-	}
+        {
+          error (0, errno, _("cannot open `%s'"), name);
+          /* Set the status to EXIT_FAILURE, even though we
+             continue to process files after a missing file.  */
+          retcode = EXIT_FAILURE;
+          return;
+        }
       push_file (fp, full_name, true);
       free (full_name);
     }
@@ -313,18 +354,19 @@ process_file (const char *name)
    '-' forces getopt_long to hand back file names as arguments to opt
    '\1', rather than reordering the command line.  */
 #ifdef ENABLE_CHANGEWORD
-#define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:W:d::eil:o:st:"
+#define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:W:d::egil:o:st:"
 #else
-#define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:d::eil:o:st:"
+#define OPTSTRING "-B:D:EF:GH:I:L:N:PQR:S:T:U:d::egil:o:st:"
 #endif
 
 int
-main (int argc, char *const *argv, char *const *envp)
+main (int argc, char *const *argv)
 {
-  macro_definition *head;	/* head of deferred argument list */
+  struct sigaction act;
+  macro_definition *head;       /* head of deferred argument list */
   macro_definition *tail;
   macro_definition *defn;
-  int optchar;			/* option character */
+  int optchar;                  /* option character */
 
   macro_definition *defines;
   bool interactive = false;
@@ -334,141 +376,186 @@ main (int argc, char *const *argv, char *const *envp)
   const char *frozen_file_to_write = NULL;
   const char *macro_sequence = "";
 
-  program_name = argv[0];
+  set_program_name (argv[0]);
   retcode = EXIT_SUCCESS;
   atexit (close_stdin);
 
   include_init ();
   debug_init ();
-#ifdef USE_STACKOVF
-  setup_stackovf_trap (argv, envp, stackovf_handler);
-#endif
+
+  /* Stack overflow and program error handling.  Ignore failure to
+     install a handler, since this is merely for improved output on
+     crash, and we should never crash ;).  We install SIGBUS and
+     SIGSEGV handlers prior to using the c-stack module; depending on
+     the platform, c-stack will then override none, SIGSEGV, or both
+     handlers.  */
+  program_error_message
+    = xasprintf (_("internal error detected; please report this bug to <%s>"),
+                 PACKAGE_BUGREPORT);
+  signal_message[SIGSEGV] = xstrdup (strsignal (SIGSEGV));
+  signal_message[SIGABRT] = xstrdup (strsignal (SIGABRT));
+  signal_message[SIGILL] = xstrdup (strsignal (SIGILL));
+  signal_message[SIGFPE] = xstrdup (strsignal (SIGFPE));
+  if (SIGBUS != SIGILL && SIGBUS != SIGSEGV)
+    signal_message[SIGBUS] = xstrdup (strsignal (SIGBUS));
+  sigemptyset (&act.sa_mask);
+  /* One-shot - if we fault while handling a fault, we want to revert
+     to default signal behavior.  */
+  act.sa_flags = SA_NODEFER | SA_RESETHAND;
+  act.sa_handler = fault_handler;
+  sigaction (SIGSEGV, &act, NULL);
+  sigaction (SIGABRT, &act, NULL);
+  sigaction (SIGILL, &act, NULL);
+  sigaction (SIGFPE, &act, NULL);
+  sigaction (SIGBUS, &act, NULL);
+  if (c_stack_action (fault_handler) == 0)
+    nesting_limit = 0;
+
+#ifdef DEBUG_STKOVF
+  /* Make it easier to test our fault handlers.  Exporting M4_CRASH=0
+     attempts a SIGSEGV, exporting it as 1 attempts an assertion
+     failure with a fallback to abort.  */
+  {
+    char *crash = getenv ("M4_CRASH");
+    if (crash)
+      {
+        if (!strtol (crash, NULL, 10))
+          ++*(int *) 8;
+        assert (false);
+        abort ();
+      }
+  }
+#endif /* DEBUG_STKOVF */
 
   /* First, we decode the arguments, to size up tables and stuff.  */
-
   head = tail = NULL;
 
   while ((optchar = getopt_long (argc, (char **) argv, OPTSTRING,
-				 long_options, NULL)) != -1)
+                                 long_options, NULL)) != -1)
     switch (optchar)
       {
       default:
-	usage (EXIT_FAILURE);
+        usage (EXIT_FAILURE);
 
       case 'B':
       case 'S':
       case 'T':
-	/* Compatibility junk: options that other implementations
-	   support, but which we ignore as no-ops and don't list in
-	   --help.  */
-	error (0, 0, "Warning: `m4 -%c' may be removed in a future release",
-	       optchar);
-	break;
+        /* Compatibility junk: options that other implementations
+           support, but which we ignore as no-ops and don't list in
+           --help.  */
+        error (0, 0, _("warning: `m4 -%c' may be removed in a future release"),
+               optchar);
+        break;
 
       case 'N':
       case DIVERSIONS_OPTION:
-	/* -N became an obsolete no-op in 1.4.x.  */
-	error (0, 0, "Warning: `m4 %s' is deprecated",
-	       optchar == 'N' ? "-N" : "--diversions");
+        /* -N became an obsolete no-op in 1.4.x.  */
+        error (0, 0, _("warning: `m4 %s' is deprecated"),
+               optchar == 'N' ? "-N" : "--diversions");
+        break;
 
       case 'D':
       case 'U':
       case 's':
       case 't':
       case '\1':
-	/* Arguments that cannot be handled until later are accumulated.  */
+      case DEBUGFILE_OPTION:
+        /* Arguments that cannot be handled until later are accumulated.  */
 
-	defn = (macro_definition *) xmalloc (sizeof (macro_definition));
-	defn->code = optchar;
-	defn->arg = optarg;
-	defn->next = NULL;
+        defn = (macro_definition *) xmalloc (sizeof (macro_definition));
+        defn->code = optchar;
+        defn->arg = optarg;
+        defn->next = NULL;
 
-	if (head == NULL)
-	  head = defn;
-	else
-	  tail->next = defn;
-	tail = defn;
+        if (head == NULL)
+          head = defn;
+        else
+          tail->next = defn;
+        tail = defn;
 
-	break;
+        break;
 
       case 'E':
-	if (! fatal_warnings)
-	  fatal_warnings = true;
-	else
-	  warning_status = EXIT_FAILURE;
-	break;
+        if (! fatal_warnings)
+          fatal_warnings = true;
+        else
+          warning_status = EXIT_FAILURE;
+        break;
 
       case 'F':
-	frozen_file_to_write = optarg;
-	break;
+        frozen_file_to_write = optarg;
+        break;
 
       case 'G':
-	no_gnu_extensions = 1;
-	break;
+        no_gnu_extensions = 1;
+        break;
 
       case 'H':
-	hash_table_size = atol (optarg);
-	if (hash_table_size == 0)
-	  hash_table_size = HASHMAX;
-	break;
+        hash_table_size = strtol (optarg, NULL, 10);
+        if (hash_table_size == 0)
+          hash_table_size = HASHMAX;
+        break;
 
       case 'I':
-	add_include_directory (optarg);
-	break;
+        add_include_directory (optarg);
+        break;
 
       case 'L':
-	nesting_limit = atoi (optarg);
-	break;
+        nesting_limit = strtol (optarg, NULL, 10);
+        break;
 
       case 'P':
-	prefix_all_builtins = 1;
-	break;
+        prefix_all_builtins = 1;
+        break;
 
       case 'Q':
-	suppress_warnings = 1;
-	break;
+        suppress_warnings = 1;
+        break;
 
       case 'R':
-	frozen_file_to_read = optarg;
-	break;
+        frozen_file_to_read = optarg;
+        break;
 
 #ifdef ENABLE_CHANGEWORD
       case 'W':
-	user_word_regexp = optarg;
-	break;
+        user_word_regexp = optarg;
+        break;
 #endif
 
       case 'd':
-	debug_level = debug_decode (optarg);
-	if (debug_level < 0)
-	  {
-	    error (0, 0, "bad debug flags: `%s'", optarg);
-	    debug_level = 0;
-	  }
-	break;
+        debug_level = debug_decode (optarg);
+        if (debug_level < 0)
+          {
+            error (0, 0, _("bad debug flags: `%s'"), optarg);
+            debug_level = 0;
+          }
+        break;
 
       case 'e':
-	error (0, 0, "Warning: `m4 -e' is deprecated, use `-i' instead");
-	/* fall through */
+        error (0, 0, _("warning: `m4 -e' is deprecated, use `-i' instead"));
+        /* fall through */
       case 'i':
-	interactive = true;
-	break;
+        interactive = true;
+        break;
+
+      case 'g':
+        no_gnu_extensions = 0;
+        break;
 
       case 'l':
-	max_debug_argument_length = atoi (optarg);
-	if (max_debug_argument_length <= 0)
-	  max_debug_argument_length = 0;
-	break;
+        max_debug_argument_length = strtol (optarg, NULL, 10);
+        if (max_debug_argument_length <= 0)
+          max_debug_argument_length = 0;
+        break;
 
       case 'o':
-	/* -o/--error-output are deprecated synonyms of --debugfile,
-	   but don't issue a deprecation warning until autoconf 2.61
-	   or later is more widely established, as such a warning
-	   would interfere with all earlier versions of autoconf.  */
-      case DEBUGFILE_OPTION:
-	/* Don't call debug_set_output here, as it has side effects.  */
-	debugfile = optarg;
-	break;
+        /* -o/--error-output are deprecated synonyms of --debugfile,
+           but don't issue a deprecation warning until autoconf 2.61
+           or later is more widely established, as such a warning
+           would interfere with all earlier versions of autoconf.  */
+        /* Don't call debug_set_output here, as it has side effects.  */
+        debugfile = optarg;
+        break;
 
       case WARN_MACRO_SEQUENCE_OPTION:
          /* Don't call set_macro_sequence here, as it can exit.
@@ -476,23 +563,23 @@ main (int argc, char *const *argv, char *const *envp)
             default regexp); --warn-macro-sequence= sets optarg to ""
             (which disables these warnings).  */
         macro_sequence = optarg;
-	break;
+        break;
 
       case VERSION_OPTION:
-	version_etc (stdout, PACKAGE, PACKAGE_NAME, VERSION, AUTHORS, NULL);
-	exit (EXIT_SUCCESS);
-	break;
+        version_etc (stdout, PACKAGE, PACKAGE_NAME, VERSION, AUTHORS, NULL);
+        exit (EXIT_SUCCESS);
+        break;
 
       case HELP_OPTION:
-	usage (EXIT_SUCCESS);
-	break;
+        usage (EXIT_SUCCESS);
+        break;
       }
 
   defines = head;
 
   /* Do the basic initializations.  */
   if (debugfile && !debug_set_output (debugfile))
-    M4ERROR ((0, errno, "cannot set debug file `%s'", debugfile));
+    M4ERROR ((warning_status, errno, "cannot set debug file `%s'", debugfile));
 
   input_init ();
   output_init ();
@@ -522,41 +609,47 @@ main (int argc, char *const *argv, char *const *envp)
       symbol *sym;
 
       switch (defines->code)
-	{
-	case 'D':
-	  {
-	    /* defines->arg is read-only, so we need a copy.  */
-	    char *macro_name = xstrdup (defines->arg);
-	    char *macro_value = strchr (macro_name, '=');
-	    if (macro_value)
-	      *macro_value++ = '\0';
-	    define_user_macro (macro_name, macro_value, SYMBOL_INSERT);
-	    free (macro_name);
-	  }
-	  break;
+        {
+        case 'D':
+          {
+            /* defines->arg is read-only, so we need a copy.  */
+            char *macro_name = xstrdup (defines->arg);
+            char *macro_value = strchr (macro_name, '=');
+            if (macro_value)
+              *macro_value++ = '\0';
+            define_user_macro (macro_name, macro_value, SYMBOL_INSERT);
+            free (macro_name);
+          }
+          break;
 
-	case 'U':
-	  lookup_symbol (defines->arg, SYMBOL_DELETE);
-	  break;
+        case 'U':
+          lookup_symbol (defines->arg, SYMBOL_DELETE);
+          break;
 
-	case 't':
-	  sym = lookup_symbol (defines->arg, SYMBOL_INSERT);
-	  SYMBOL_TRACED (sym) = true;
-	  break;
+        case 't':
+          sym = lookup_symbol (defines->arg, SYMBOL_INSERT);
+          SYMBOL_TRACED (sym) = true;
+          break;
 
-	case 's':
-	  sync_output = 1;
-	  break;
+        case 's':
+          sync_output = 1;
+          break;
 
-	case '\1':
-	  seen_file = true;
+        case '\1':
+          seen_file = true;
           process_file (defines->arg);
-	  break;
+          break;
 
-	default:
-	  M4ERROR ((0, 0, "INTERNAL ERROR: bad code in deferred arguments"));
-	  abort ();
-	}
+        case DEBUGFILE_OPTION:
+          if (!debug_set_output (defines->arg))
+            M4ERROR ((warning_status, errno, "cannot set debug file `%s'",
+                      debugfile ? debugfile : _("stderr")));
+          break;
+
+        default:
+          M4ERROR ((0, 0, "INTERNAL ERROR: bad code in deferred arguments"));
+          abort ();
+        }
 
       next = defines->next;
       free (defines);
